/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "editablecmb.h"
#include <QApplication>
#include "framedlg.h"
#include <QFontMetrics>
#include "mainwnd.h"
#include "ui_mainwnd.h"
#include "painthidden.h"

editableCmb::editableCmb(QWidget *parent, abstractUICCtrl *uiCtrl) :
    QComboBox(parent)
{
    underMovement = false;
    this->parentUICtrl = uiCtrl;
    this->setFocusPolicy(Qt::StrongFocus);
    this->setContextMenuPolicy(Qt::NoContextMenu);
    this->isHidden = false;
}


editableCmb::editableCmb(const QString & text, QWidget * parent, abstractUICCtrl *uiCtrl):
    QComboBox(parent)
{
    this->setEditText(text);
    underMovement = false;
    this->parentUICtrl=uiCtrl;
    this->setFocusPolicy(Qt::StrongFocus);
    this->setContextMenuPolicy(Qt::NoContextMenu);
    this->isHidden = false;
}
void editableCmb::setParents(QWidget *parent, abstractUICCtrl *uiCtrl)
{
    setParent(parent);
    this->parentUICtrl = uiCtrl;
}

void editableCmb::setHidden(bool isHidden){
    this->isHidden = isHidden;
}

editableCmb::~editableCmb()
{

}

int editableCmb::cordToGrid(int v)
{
    int ret;
    ret = v / 5;
    ret = ret * 5;
    return ret;
}

void editableCmb::mouseReleaseEvent(QMouseEvent *)
{
    underMovement = false;
    this->parentUICtrl->updateAdjPoints();
}

void editableCmb::mousePressEvent(QMouseEvent * event)
{
    childWndDlg *wnd = this->parentUICtrl->getParentWnd();

    underMovement = true;
    moveXInit = this->cordToGrid(event->x());
    moveYInit = this->cordToGrid(event->y());

    if(!wnd->isSelected(this->parentUICtrl))
      wnd->addToSelection(this->parentUICtrl);
}

void editableCmb::mouseMoveEvent(QMouseEvent * event)
{
    QRect sel;
    childWndDlg *wnd = this->parentUICtrl->getParentWnd();
    int dx;
    int dy;
    int pWidth = ((QWidget * )parent())->width();
    int pHeight = ((QWidget * )parent())->height();
    sel = wnd->sizeSelection();
    if(underMovement)
    {
        dx = this->cordToGrid(event->x()) - moveXInit;
        dy = this->cordToGrid(event->y()) - moveYInit;
        if((sel.left() + dx >= 0) &&
           (sel.top() + dy >= 0) &&
           (sel.right() + dx <= pWidth) &&
           (sel.bottom() + dy <= pHeight))
          wnd->moveSelection(dx, dy);
    }
}


void editableCmb::moveEvent ( QMoveEvent * event )
{
    QComboBox::moveEvent(event);
    this->parentUICtrl->updateAdjPoints();    
    this->parentWidget()->update();
    mainWnd::getPropPan()->updatePropertiesValues();
}

void editableCmb::resizeEvent ( QResizeEvent * event )
{
    QComboBox::resizeEvent(event);
    this->parentUICtrl->updateAdjPoints();
    this->parentWidget()->update();
    mainWnd::getPropPan()->updatePropertiesValues();
}

void editableCmb::keyPressEvent(QKeyEvent *e)
{
    if (e->type() == QEvent::KeyPress)
    {
        QKeyEvent* newEvent = new QKeyEvent(QEvent::KeyPress,e->key(), e->modifiers ());
        qApp->postEvent (this->parent(), newEvent, 0);
    }
}

void editableCmb::mouseDoubleClickEvent(QMouseEvent *event __attribute__ ((unused))){
    if(this->parentUICtrl->getParentWnd()->getTab()){
        this->parentUICtrl->getParentWnd()->getTab()->setCurrentIndex(this->parentUICtrl->getParentWnd()->getTabSrc());
        this->parentUICtrl->properties.getPropByName("GenerateCallBack")->setValue("true");
        this->parentUICtrl->getParentWnd()->getProPanel()->updatePropertiesValues();
    }
}

void editableCmb::paintEvent(QPaintEvent *e){
    QComboBox::paintEvent(e);
    if(this->isHidden)
        paintHidden ph(this);
}
