/****************************************************************************
Copyright 2015-2020, Enrique Sergio Burgos

This file is part of GUI Editor for Octave.

GUI Editor for Octave is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GUI Editor is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GUI Editor.  If not, see <http://www.gnu.org/licenses/>.
****************************************************************************/
#include "guiproject.h"
#include <QMenu>
#include <QMessageBox>
#include <QObject>
#include <QMdiSubWindow>
#include "dlgnewfile.h"
#include "ui_dlgnewfile.h"
#include "mainwnd.h"
#include "ui_mainwnd.h"

#include "childwnddlg.h"
#include "childwndmfile.h"
#include "childwndimg.h"
#include "childwndtextfile.h"

#include "ui_wdgmfileeditor.h"
#include "wdgmfileeditor.h"

#include "framedlg.h"
#include <QDir>

#include <QFile>
#include <QTextStream>
#include "storagemanager.h"

#include <QMessageBox>

#include "dlgprjprop.h"
#include "ui_dlgprjprop.h"

#include <QFileDialog>
#include <QStandardPaths>
#include <QDateTime>

#include "dlgpkggen.h"
#include "ui_dlgpkggen.h"

#include "dlgsetupargs.h"
#include "ui_dlgsetupargs.h"

#include "dlgconfirmonclose.h"

#include <QMessageBox>
#include <QDebug>

#include "dlgfilechanged.h"
#include "dlgfiledeleted.h"

treeItemGroup::treeItemGroup(guiProject * p, QTreeWidget * parent):
    abstractPrjTreeItem(parent)
{
    this->setText(0, p->name());
    this->setIcon(0, QPixmap(":/img/projectItem"));
    this->prj = p;
}

void treeItemGroup::activate()
{

}

void treeItemGroup::populateMenu(QMenu &m)
{
    m.addAction(tr("Add a new dialog"), this, SLOT(actAddNewDlg(bool)));
    m.addAction(tr("Add a new m file"), this, SLOT(actAddNewMFile(bool)));
    m.addAction(tr("Add a new image"), this, SLOT(actAddNewImage(bool)));
    m.addSeparator();
    m.addAction(tr("Export project"), this, SLOT(actExport(bool)));
    m.addAction(tr("Generate octave package"), this, SLOT(actGenPkg(bool)));
    m.addSeparator();
    m.addAction(tr("Save project"), this, SLOT(actSave(bool)));
    m.addAction(tr("Close project"), this, SLOT(actClose(bool)));
    m.addSeparator();
    m.addAction(tr("Run project"), this, SLOT(actRun(bool)));
    m.addAction(tr("Set run arguments"), this, SLOT(actRunArguments(bool)));
    m.addSeparator();
    m.addAction(tr("Properties"), this, SLOT(actProjectProperties(bool)));
}

void treeItemGroup::selectProject()
{
    QMessageBox::information(0, "Cuidado", "Select project");
    /*   mainWnd::getPrjWnd()->setActiveProject(this->prj);*/
}

void treeItemGroup::actAddNewDlg(bool checked __attribute__((unused)))
{
    int tabNro;
    QMdiArea * mdiArea;
    wdgSrcEditor * srcEdit;
    QTabWidget * newTab;
    dlgNewFile wnd(this->prj);
    if(wnd.exec() == QDialog::Accepted)
    {

        newTab = new QTabWidget(this->getPrj()->getProjectWidget()->ui->docTabs);
        newTab->setTabPosition(QTabWidget::South);
        tabNro = this->getPrj()->getProjectWidget()->ui->docTabs->addTab(newTab, wnd.ui->leFileName->text());
        this->getPrj()->getProjectWidget()->ui->docTabs->setCurrentIndex(tabNro);
        srcEdit = new wdgSrcEditor(newTab);
        tabNro = newTab->addTab(srcEdit , tr("[&source]"));
        mdiArea = new QMdiArea(newTab);
        mdiArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        mdiArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
        childWndDlg * dlg = new childWndDlg(nullptr,
                                            mainWnd::getPropPan(),
                                            this->prj);

        mdiArea->addSubWindow(dlg);
        dlg->setSrcEditor(srcEdit);
        dlg->show();
        dlg->setTab(newTab, tabNro);
        tabNro = newTab->addTab(mdiArea, tr("[&design]"));
        newTab->setCurrentIndex(tabNro);

        dlg->getFrame()->setCtrlName(wnd.ui->leFileName->text());

        dlg->save(this->prj->guiPath() + QDir::separator() + wnd.ui->leFileName->text() + ".xml");

        dlg->setGeometry(1, 1, 250, 100);
        dlg->getFrame()->updateSrcWndCtrls();
        dlg->getFrame()->setCanChangeName(false);
        this->prj->items.push_back(dlg);
        this->getPrj()->getProjectWidget()->updateTreeNames();
    }

}

void treeItemGroup::actAddNewMFile(bool checked __attribute__((unused)))
{
    QString fcnTemp;
    int tabNro;
    dlgNewFile wnd(this->prj);
    if(wnd.exec() == QDialog::Accepted)
    {
        childWndMFile * dlg = new childWndMFile(this->getPrj()->getProjectWidget()->ui->docTabs,
                                                this->prj);
        dlg->save(this->prj->fcnPath() + QDir::separator() + wnd.ui->leFileName->text() + ".m");
        tabNro = this->getPrj()->getProjectWidget()->ui->docTabs->addTab(dlg, dlg->getName());
        this->getPrj()->getProjectWidget()->ui->docTabs->setCurrentIndex(tabNro);
        QFile f(":/template/templates/fcn.m");
        f.open(QIODevice::ReadOnly);
        fcnTemp = f.readAll();
        f.close();
        fcnTemp.replace("<fcnName>",
                        wnd.ui->leFileName->text());
        dlg->getSrcEditor()->ui->srcEditor->setText(fcnTemp);
        this->prj->items.push_back(dlg);
        this->getPrj()->getProjectWidget()->updateTreeNames();
    }
}

void treeItemGroup::actAddNewImage(bool checked __attribute__((unused)))
{
    QString basePath;
    if(QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation).count() > 0)
        basePath = QStandardPaths::standardLocations(QStandardPaths::DocumentsLocation)[0];
    else
        basePath = QApplication::applicationDirPath();
    QString imgFile = QFileDialog::getOpenFileName(mainWnd::getMainWnd(),
                                                   tr("Select image to add"),
                                                   basePath,
                                                   tr("PNG Files (*.png);;SVG Files (*.svg);;All files (*.*);;"));
    QString outFile;
    if(!imgFile.isNull())
    {
        QFileInfo fileInfo(imgFile);
        outFile = this->getPrj()->imgPath() + QDir::separator() + fileInfo.fileName();
        QFile::copy(imgFile, outFile);
        childWndImg * dlg = new childWndImg(this->getPrj()->getProjectWidget()->ui->docTabs, this->prj, outFile);
        dlg->load(outFile);
        dlg->setWindowTitle(dlg->getName());
        this->prj->items.push_back(dlg);
        this->getPrj()->getProjectWidget()->updateTreeNames();
    }
}

void treeItemGroup::actRun(bool checked __attribute__((unused)))
{
    this->prj->run();
}

void treeItemGroup::actRunArguments(bool checked __attribute__((unused))){

    dlgSetupArgs args(0);
    args.setArguments(this->prj->runArguments());
    if(args.exec() == QDialog::Accepted)
        this->prj->setRunArguments(args.getArguments());
}

void treeItemGroup::actExport(bool checked __attribute__((unused)))
{
    this->prj->exportPrj();
}

void treeItemGroup::actGenPkg(bool checked __attribute__((unused)))
{
    this->prj->generatePkg();
}

void treeItemGroup::actProjectProperties(bool checkded __attribute__((unused)))
{
    int k;
    int vA, vB, vC;
    dlgPrjProp wnd(mainWnd::getMainWnd());

    wnd.ui->cbDefBasePath->setChecked(this->prj->defBasePath());
    wnd.ui->cbDefBasePath->setText(tr("Define ") + "_" + this->prj->name() + "BasePath");

    wnd.ui->cbDefImgPath->setChecked(this->prj->defImgPath());
    wnd.ui->cbDefImgPath->setText(tr("Define ") + "_" + this->prj->name() + "ImgPath");
    this->prj->version(vA, vB, vC);
    wnd.ui->sbVerA->setValue(vA);
    wnd.ui->sbVerB->setValue(vB);
    wnd.ui->sbVerC->setValue(vC);
    wnd.ui->cbDisWarn->setChecked(this->prj->disableAllWarnings);
    wnd.ui->lstWarnMsg->addItems(this->prj->disabledWarningMessages);
    if(wnd.exec())
    {
        this->prj->setDefBasePath(wnd.ui->cbDefBasePath->isChecked());
        this->prj->setDefImgPath(wnd.ui->cbDefImgPath->isChecked());
        vA = wnd.ui->sbVerA->value();
        vB = wnd.ui->sbVerB->value();
        vC = wnd.ui->sbVerC->value();
        this->prj->setVersion(vA, vB, vC);
        this->prj->disableAllWarnings = wnd.ui->cbDisWarn->isChecked();
        this->prj->disabledWarningMessages.clear();
        for(k = 0; k < wnd.ui->lstWarnMsg->count(); k++){
            this->prj->disabledWarningMessages.append(wnd.ui->lstWarnMsg->item(k)->text());
        }
    }
}

void treeItemGroup::actSave(bool checked __attribute__((unused)))
{
    this->prj->save();
}

void treeItemGroup::actClose(bool checked __attribute__((unused)))
{
    this->getPrj()->close();
}

void guiProject::copyFolder(QString src, QString dst)
{
    QDir dir(src);
    if (! dir.exists())
        return;

    foreach (QString d, dir.entryList(QDir::Dirs | QDir::NoDotAndDotDot)) {
        QString dst_path = dst + QDir::separator() + d;
        dir.mkpath(dst_path);
        copyFolder(src+ QDir::separator() + d, dst_path);
    }

    foreach (QString f, dir.entryList(QDir::Files)) {
        if(QFile::exists(dst + QDir::separator() + f))
            QFile::remove(dst + QDir::separator() + f);
        QFile::copy(src + QDir::separator() + f, dst + QDir::separator() + f);
    }
}

void guiProject::closeEvent(QCloseEvent *event){
    bool isModified = false;
    dlgConfirmOnClose  wnd;
    QList<projectItem *>::iterator it;
    for(it = this->items.begin(); it != this->items.end(); it++)
        isModified = isModified || (*it)->isModified();
    if(isModified){
        wnd.setProjectName(this->vName);
        switch(wnd.exec()){
        case cancelAndReturn:
            event->ignore();
            break;
        case saveAndClose:
            this->save();
            event->accept();
            break;
        case dontSaveAndClose:
            event->accept();
            break;
        }
    }
}

guiProject::guiProject(QString path, QString name, QWidget *parent, Qt::WindowFlags flags):QMdiSubWindow(parent, flags)
{
    fsw = new QFileSystemWatcher(this);

    vPath = path;
    vName = name;
    this->setWindowTitle(vName);
    QDir().mkdir(path);
    QDir prjPath(path);
    if(!prjPath.exists("export"))
        prjPath.mkdir("export");

    if(!prjPath.exists("fcn"))
        prjPath.mkdir("fcn");


    if(!prjPath.exists("gui"))
        prjPath.mkdir("gui");

    if(!prjPath.exists("img"))
        prjPath.mkdir("img");

    if(!prjPath.exists("other"))
        prjPath.mkdir("other");

    if(!prjPath.exists("pkg"))
        prjPath.mkdir("pkg");

    if(!prjPath.exists("run"))
        prjPath.mkdir("run");

    if(!prjPath.exists("libs"))
        prjPath.mkdir("libs");

    vMainDlgName = NULL;
    this->vDefBasePath = true;
    this->vDefImgPath = true;
    wdgPrj = new   wdgProject(this, this);
    this->setWidget(wdgPrj);

    this->enableFSWatch();
    this->disableAllWarnings = false;
}

guiProject::~guiProject()
{
    disableFSWatch();
    delete fsw;
    QList<projectItem *>::iterator it;
    for(it = this->items.begin(); it != this->items.end(); it++)
        delete (*it);
    this->items.clear();
}

void guiProject::setVersion(int A, int B, int C)
{
    vVerA = A;
    vVerB = B;
    vVerC = C;
}

QString guiProject::name()
{
    return this->vName;
}


QString guiProject::path()
{
    return this->vPath;
}

QString guiProject::fullPath()
{
    return this->vPath + QDir::separator() + this->vName;
}


bool guiProject::defBasePath()
{
    return this->vDefBasePath;
}

bool guiProject::defImgPath()
{
    return this->vDefImgPath;
}


void guiProject::setDefBasePath(bool v)
{
    this->vDefBasePath = v;
}

void guiProject::setDefImgPath(bool v)
{
    this->vDefImgPath = v;
}

QString guiProject::exportPath()
{
    return path() + QDir::separator() + "export";
}

QString guiProject::fcnPath()
{
    return path() + QDir::separator() + "fcn";
}

QString guiProject::guiPath()
{
    return path() + QDir::separator() + "gui";
}

QString guiProject::imgPath()
{
    return path() + QDir::separator() + "img";
}

QString guiProject::otherPath()
{
    return path() + QDir::separator() + "other";
}

QString guiProject::pkgPath()
{
    return path() + QDir::separator() + "pkg";
}

QString guiProject::runPath()
{
    return path() + QDir::separator() + "run";
}

QString guiProject::libsPath()
{
    return path() + QDir::separator() + "libs";
}

void guiProject::version(int &A, int &B, int &C)
{
    A = vVerA;
    B = vVerB;
    C = vVerC;
}

void guiProject::setMainDlg(QString dlgName)
{
    QList<projectItem *>::iterator it;
    bool finded = false;
    for(it = items.begin(); (it != items.end()) && (!finded); it++)
    {
        if ((*it)->getName() == dlgName)
        {
            vMainDlgName = dynamic_cast<projectItem *>((*it));
            finded = true;
        }
    }
    this->wdgPrj->updateTreeNames();
}

void guiProject::setMainDlg(projectItem *dlg)
{
    this->vMainDlgName = dlg;
    this->wdgPrj->updateTreeNames();
}

projectItem * guiProject::getMainDlg()
{
    return this->vMainDlgName;
}

void guiProject::setSearchPath(QStringList searchPath)
{
    vSearchPath = searchPath;
}

QStringList guiProject::searchPath()
{
    return vSearchPath;
}



void guiProject::update(bool openFirst)
{
    QDir guiFiles(this->guiPath());
    QString guiFile;
    QStringList fileList;
    QFileInfo fileInfo;
    int i;

    this->wdgPrj->ui->prjItems->clear();
    fileList = guiFiles.entryList(QDir::NoDotAndDotDot | QDir::Files);

    for(i = 0; i < fileList.count(); i++)
    {
        guiFile = fileList[i];
        childWndDlg * dlg = new childWndDlg(mainWnd::getMainWnd(), mainWnd::getPropPan(), this);
        storageManager::loadDlgFromFile(dlg, this->guiPath() + QDir::separator()  + guiFile);
        dlg->getFrame()->setCanChangeName(false);
        fileInfo.setFile(this->guiPath() + QDir::separator()  + guiFile);
        dlg->setLastModified(fileInfo.lastModified());
        items.push_back(dlg);
    }

    QDir mFiles(this->fcnPath());
    QString mFile;
    fileList = mFiles.entryList(QDir::NoDotAndDotDot | QDir::Files);
    for(i = 0; i < fileList.count(); i++)
    {
        mFile = fileList[i];
        childWndMFile  * dlg = new childWndMFile(mainWnd::getMainWnd(), this);
        QFile f(this->fcnPath() + QDir::separator() + mFile);
        f.open(QIODevice::ReadOnly | QIODevice::Text);
        dlg->getSrcEditor()->ui->srcEditor->setText(f.readAll());
        dlg->documentHaveName = true;
        dlg->documentName = this->fcnPath() + QDir::separator() + mFile;
        dlg->setWindowTitle(dlg->getName());
        f.close();
        fileInfo.setFile(this->fcnPath() + QDir::separator() + mFile);
        dlg->setLastModified(fileInfo.lastModified());
        items.push_back(dlg);
    }

    QDir imgFiles(this->imgPath());
    QString imgFile;
    fileList = imgFiles.entryList(QDir::NoDotAndDotDot | QDir::Files);
    for(i = 0; i < fileList.count(); i++)
    {
        imgFile = fileList[i];
        childWndImg  * dlg = new childWndImg(mainWnd::getMainWnd(), this, this->imgPath() + QDir::separator()  + imgFile);

        dlg->documentHaveName = true;
        dlg->documentName = this->imgPath() + QDir::separator()  + imgFile;        
        dlg->setWindowTitle(dlg->getName());
        fileInfo.setFile(this->imgPath() + QDir::separator()  +imgFile);
        dlg->setLastModified(fileInfo.lastModified());
        items.push_back(dlg);
    }

    QDir otherFiles(this->otherPath());
    QString otherFile;
    fileList = otherFiles.entryList(QDir::NoDotAndDotDot | QDir::Files);
    for(i = 0; i < fileList.count(); i++)
    {
        otherFile = fileList[i];
        childWndTextFile * dlg = new childWndTextFile(mainWnd::getMainWnd(), this);
        dlg->documentHaveName = true;
        dlg->documentName = this->otherPath() + QDir::separator() + otherFile;
        dlg->load(dlg->documentName);
        dlg->setWindowTitle(dlg->getName());
        fileInfo.setFile(this->otherPath() + QDir::separator() + otherFile);
        dlg->setLastModified(fileInfo.lastModified());
        items.push_back(dlg);
    }
    this->wdgPrj->updateTreeNames(openFirst);
}

void guiProject::exportPrj()
{
    int k;
    QString warnMsg;
    this->save();
    QString exportPath = this->exportPath();
    QStringList guiLibsFiles;
    QDir addDir(exportPath);
    addDir.mkdir("fcn");
    addDir.mkdir("img");
    addDir.mkdir("guilib");
    addDir.mkdir("wnd");

    guiLibsFiles << "_loadIcon.m";

    for(k = 0; k < guiLibsFiles.count(); k++){
        if(QFile::exists(this->exportPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]))
            QFile::remove(this->exportPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]);
        QFile::copy(":/guilib/guilib/" + guiLibsFiles[k],
                    this->exportPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]);
        QFile(this->exportPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]).setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner | QFileDevice::ReadGroup);
    }

    if(addDir.exists("libs"))
    {
        QDir rmLibs(addDir);
        rmLibs.cd("libs");
        rmLibs.removeRecursively();
    }
    addDir.mkdir("libs");
    copyFolder(this->path() + QDir::separator() + "libs", this->exportPath() + QDir::separator() + "libs");

    QList<projectItem *>::iterator it;

    for(it = items.begin(); it != items.end(); it++)
    {
        (*it)->exportAsScript(exportPath);
        if((*it) == this->vMainDlgName)
        {
            QFile f;
            f.setFileName(exportPath + QDir::separator() + this->name() + ".m");
            f.open(QIODevice::WriteOnly | QIODevice::Text);
            QTextStream txt(&f);
            txt.setCodec(mainWnd::getAppConfig()->getOutputCodec().toStdString().c_str());
            txt << "## -*- texinfo -*-" << endl;
            txt << "## @deftypefn  {} {@var{ret} =} " + this->name() + " (varargin)" << endl;
            txt << "##" << endl;
            txt << "## Create and show the main dialog, return a struct as representation of dialog." << endl;
            txt << "##" << endl;
            txt << "## @end deftypefn" << endl;
            txt << "function ret = " + this->name() + "(varargin)" << endl;
            /*   Warning messages supressions   */
            if(this->disableAllWarnings)
                txt << "  warning(\"off\");" << endl;
            for(k = 0; k < this->disabledWarningMessages.count(); k++){
                warnMsg = this->disabledWarningMessages[k].replace("\"", "\\\"");
                txt << "  warning(\"off\", \"" + warnMsg + "\");" << endl;
            }
            /* End warning messages supressions */
            txt << "  [dir, name, ext] = fileparts( mfilename('fullpathext') );" << endl;
            if(mainWnd::getAppConfig()->getForceOutputCodec()){
                txt << "  _initialCodec = __mfile_encoding__;"  << endl;
                txt << "  __mfile_encoding__(\"" + mainWnd::getAppConfig()->getOutputCodec() + "\");"  << endl;
                txt << "  clear (\"functions\");"  << endl;
            }

            if(this->defBasePath())
                txt << "  global _" << this->name() << "BasePath = dir;" << endl;
            if(this->defImgPath())
                txt << "  global _" << this->name() << "ImgPath = [dir filesep() 'img'];" << endl;
            txt << "  addpath( [dir filesep() \"img\" ]);" << endl;
            txt << "  addpath( [dir filesep() \"fcn\" ]);" << endl;
            txt << "  addpath( [dir filesep() \"libs\" ]);" << endl;
            txt << "  addpath( [dir filesep() \"wnd\" ]);" << endl;
            txt << "  addpath([dir filesep() \"guilib\" ]);" << endl;
            txt << "  ret = " << (*it)->getName() << "(varargin{:});" << endl;
            if(mainWnd::getAppConfig()->getForceOutputCodec()){
                txt << "  __mfile_encoding__(_initialCodec);"  << endl;
            }

            txt << "end" << endl;
            f.close();
        }
    }
    QMessageBox::information(mainWnd::getMainWnd(), QObject::tr("Export project"), QObject::tr("The project was exported to folder ")+ exportPath);

}

void guiProject::run()
{
    int k;
    QString warnMsg;
    QStringList paths;
    QStringList guiLibsFiles;
    QDir addDir(this->runPath());
    this->save();
    QString runPath = this->runPath();
    QList<projectItem *>::iterator it;
    addDir.mkdir("fcn");
    addDir.mkdir("guilib");
    addDir.mkdir("img");
    addDir.mkdir("wnd");
    guiLibsFiles << "_loadIcon.m";
    for(k = 0; k < guiLibsFiles.count(); k++){
        if(QFile::exists(this->runPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]))
            QFile::remove(this->runPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]);
        QFile::copy(":/guilib/guilib/" + guiLibsFiles[k],
                    this->runPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]);
        QFile(this->runPath() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]).setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner | QFileDevice::ReadGroup);
    }
    if(addDir.exists("libs"))
    {
        QDir rmLibs(addDir);
        rmLibs.cd("libs");
        rmLibs.removeRecursively();
    }

    addDir.mkdir("libs");
    copyFolder(this->path() + QDir::separator() + "libs", this->runPath() + QDir::separator() + "libs");

    bool okRun = false;
    for(it = items.begin(); it != items.end(); it++)
        if((*it) == this->vMainDlgName)
            okRun = true;
    if(okRun)
    {
        for(it = items.begin(); it != items.end(); it++)
        {
            (*it)->exportAsScript(runPath);
            if((*it) == this->vMainDlgName)
            {
                QFile f;
                f.setFileName(this->runPath() + QDir::separator() + "runApp.m");
                f.open(QIODevice::WriteOnly | QIODevice::Text);
                QTextStream txt(&f);
                txt.setCodec(mainWnd::getAppConfig()->getOutputCodec().toStdString().c_str());
                txt << "function ret = runApp(varargin)" << endl;
                /*   Warning messages supressions   */
                if(this->disableAllWarnings)
                    txt << "  warning(\"off\");" << endl;
                for(k = 0; k < this->disabledWarningMessages.length(); k++){
                    warnMsg = this->disabledWarningMessages[k].replace("\"", "\\\"");
                    txt << "  warning(\"off\", \"" + warnMsg + "\");" << endl;
                }
                /* End warning messages supressions */
                txt << "  [dir, name, ext] = fileparts( mfilename('fullpathext') );" << endl;
                if(mainWnd::getAppConfig()->getForceOutputCodec()){
                    txt << "  _initialCodec = __mfile_encoding__;"  << endl;
                    txt << "  __mfile_encoding__(\"" + mainWnd::getAppConfig()->getOutputCodec() + "\");"  << endl;
                    txt << "  clear (\"functions\");"  << endl;
                }
                if(this->defBasePath())
                    txt << "  global _" << this->name() <<"BasePath = dir;" << endl;
                if(this->defImgPath())
                    txt << "  global _" << this->name() <<"ImgPath = [dir filesep() 'img'];" << endl;

                txt << "  addpath([dir filesep() \"libs\" ]);" << endl;
                txt << "  addpath([dir filesep() \"fcn\" ]);" << endl;
                txt << "  addpath([dir filesep() \"wnd\" ]);" << endl;
                txt << "  addpath([dir filesep() \"guilib\" ]);" << endl;
                txt << "  waitfor(" << (*it)->getName() << "(" << this->runArguments() << ").figure);" << endl;
                if(mainWnd::getAppConfig()->getForceOutputCodec()){
                    txt << "  __mfile_encoding__(_initialCodec);"  << endl;
                }

                txt << "end" << endl;
                f.close();
            }
        }
        paths.append(this->runPath());
        mainWnd::getMainWnd()->runScript(this->runPath() + QDir::separator() + "runApp.m",
                                         paths,
                                         this);
    }
    else
        QMessageBox::information(mainWnd::getMainWnd(), QObject::tr("Not defined a main dialog"), QObject::tr("To run a project, you must define a main dialog. This will be the first dialog showed. If there is no dialog in project, add one. If have some dialog, mark as main dialog (left click over them in the project mannager, and select \"set as main dialog\")"));
}

void guiProject::setRunArguments(QString args){
    this->args = args;
}

QString guiProject::runArguments(void){
    return this->args;
}

void guiProject::generatePkg()
{        
    int k;
    QString warnMsg;
    QStringList guiLibsFiles;
    QString line;
    QStringList varDef;
    QString strVer = QString::number(this->vVerA) + "." + QString::number(this->vVerB) + "." + QString::number(this->vVerC);
    this->save();
    if((!mainWnd::getMainWnd()->getAppConfig()->gzipPathOk()) ||
            (!mainWnd::getMainWnd()->getAppConfig()->tarPathOk()))
    {
        QMessageBox::information(mainWnd::getMainWnd() ,
                                 QObject::tr("Generate octave package"),
                                 QObject::tr("To generate a package from this project, you must configure the local path to gzip and tar program from File/Properties menu. This applications can be found in /bin or /usr/bin folder in GNU Linux or must be installed as external program in MS Windows."));
    }
    else
    {
        /* make the package base folder */
        QDir pkgFolder (this->pkgPath());
        if(!pkgFolder.exists(this->name()))
            pkgFolder.mkdir(this->name());
        pkgFolder.cd(this->name());

        /* Check if have COPYING, INDEX and DESCRIPTION files*/
        bool licFile = false;
        bool inxFile = false;
        bool descFile = false;
        bool pkgAddFile = false;
        bool pkgDelFile = false;

        QList<projectItem *>::iterator it;
        for(it = this->items.begin(); it != this->items.end(); it++)
        {
            if((*it)->getName() == "COPYING")
                licFile = true;
            else if((*it)->getName() == "INDEX")
                inxFile = true;
            else if((*it)->getName() == "DESCRIPTION")
                descFile = true;
            else if((*it)->getName() == "PKG_ADD")
                pkgAddFile = true;
            else if((*it)->getName() == "PKG_DEL")
                pkgDelFile = true;
        }
        if(!(licFile && inxFile && descFile && pkgAddFile && pkgDelFile))
        {
            QMessageBox::information(mainWnd::getMainWnd() ,
                                     QObject::tr("Generate octave package"),
                                     QObject::tr("A octave package must have special texts files with license information, details about of package and other data. Not all files are finded in project, basic versions of them are created and added."));

            if(!licFile) /* Add a basic license file */
            {
                QFile::copy(":/template/templates/COPYING", this->otherPath() + QDir::separator() + "COPYING");
                QFile(this->otherPath() + QDir::separator() + "COPYING").setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner | QFileDevice::ReadGroup);

                childWndTextFile * dlg = new childWndTextFile(this->getProjectWidget()->ui->docTabs, this);
                dlg->documentHaveName = true;
                dlg->documentName = this->otherPath() + QDir::separator() + "COPYING";
                dlg->load(dlg->documentName);
                dlg->setWindowTitle(dlg->getName());
                items.push_back(dlg);
            }

            if(!pkgAddFile)
            {
                QFile::copy(":/template/templates/PKG_ADD", this->otherPath() + QDir::separator() + "PKG_ADD");
                QFile(this->otherPath() + QDir::separator() + "PKG_ADD").setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner | QFileDevice::ReadGroup);

                childWndTextFile * dlg = new childWndTextFile(this->getProjectWidget()->ui->docTabs, this);
                dlg->documentHaveName = true;
                dlg->documentName = this->otherPath() + QDir::separator() + "PKG_ADD";
                dlg->load(dlg->documentName);
                dlg->setWindowTitle(dlg->getName());
                items.push_back(dlg);
            }

            if(!pkgDelFile)
            {
                QFile::copy(":/template/templates/PKG_DEL", this->otherPath() + QDir::separator() + "PKG_DEL");
                QFile(this->otherPath() + QDir::separator() + "PKG_DEL").setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner | QFileDevice::ReadGroup);

                childWndTextFile * dlg = new childWndTextFile(this->getProjectWidget()->ui->docTabs, this);
                dlg->documentHaveName = true;
                dlg->documentName = this->otherPath() + QDir::separator() + "PKG_DEL";
                dlg->load(dlg->documentName);
                dlg->setWindowTitle(dlg->getName());
                items.push_back(dlg);
            }

            if(!inxFile) /* Add a basic index file */
            {
                QFile inxSrc(":/template/templates/INDEX");
                QFile inxDst(this->otherPath() + QDir::separator() + "INDEX");

                inxSrc.open(QIODevice::ReadOnly | QIODevice::Text);
                inxDst.open(QIODevice::WriteOnly | QIODevice::Text);

                QTextStream inxSrcTxt(&inxSrc);
                QTextStream inxDstTxt(&inxDst);
                inxDstTxt.setCodec(mainWnd::getAppConfig()->getOutputCodec().toStdString().c_str());

                while(!inxSrcTxt.atEnd())
                {
                    line = inxSrcTxt.readLine();
                    line.replace("<pkgName>", this->name());
                    inxDstTxt << line << endl;
                }

                inxSrc.close();
                inxDst.close();

                childWndTextFile * dlg = new childWndTextFile(this->getProjectWidget()->ui->docTabs, this);
                dlg->documentHaveName = true;
                dlg->documentName = this->otherPath() + QDir::separator() + "INDEX";
                dlg->load(dlg->documentName);
                dlg->setWindowTitle(dlg->getName());
                items.push_back(dlg);
            }

            if(!descFile) /* Add a basic description file */
            {
                QDateTime now(QDateTime::currentDateTime());
                QFile dscSrc(":/template/templates/DESCRIPTION");
                QFile dscDst(this->otherPath() + QDir::separator() + "DESCRIPTION");

                dscSrc.open(QIODevice::ReadOnly | QIODevice::Text);
                dscDst.open(QIODevice::WriteOnly | QIODevice::Text);

                QTextStream dscSrcTxt(&dscSrc);
                QTextStream dscDstTxt(&dscDst);
                dscDstTxt.setCodec(mainWnd::getAppConfig()->getOutputCodec().toStdString().c_str());

                while(!dscSrcTxt.atEnd())
                {
                    line = dscSrcTxt.readLine();
                    line.replace("<pkgName>", this->name());
                    line.replace("<pkgVer>", strVer);
                    line.replace("<now>", now.toString("dd/MM/yyyy"));
                    dscDstTxt << line << endl;
                }

                dscSrc.close();
                dscDst.close();

                childWndTextFile * dlg = new childWndTextFile(this->getProjectWidget()->ui->docTabs, this);
                dlg->documentHaveName = true;
                dlg->documentName = this->otherPath() + QDir::separator() + "DESCRIPTION";
                dlg->load(dlg->documentName);
                dlg->setWindowTitle(dlg->getName());
                items.push_back(dlg);
            }
            this->getProjectWidget()->updateTreeNames();
        }
        /* The project have all nedded files*/
        if(QFile::exists(pkgFolder.path() + QDir::separator() + "COPYING"))
            QFile::remove(pkgFolder.path() + QDir::separator() + "COPYING");
        QFile::copy(this->otherPath() + QDir::separator() + "COPYING",
                    pkgFolder.path() + QDir::separator() + "COPYING");

        if(QFile::exists(pkgFolder.path() + QDir::separator() + "DESCRIPTION"))
            QFile::remove(pkgFolder.path() + QDir::separator() + "DESCRIPTION");
        QFile::copy(this->otherPath() + QDir::separator() + "DESCRIPTION",
                    pkgFolder.path() + QDir::separator() + "DESCRIPTION");

        if(QFile::exists(pkgFolder.path() + QDir::separator() + "INDEX"))
            QFile::remove(pkgFolder.path() + QDir::separator() + "INDEX");
        QFile::copy(this->otherPath() + QDir::separator() + "INDEX",
                    pkgFolder.path() + QDir::separator() + "INDEX");

        if(QFile::exists(pkgFolder.path() + QDir::separator() + "PKG_ADD"))
            QFile::remove(pkgFolder.path() + QDir::separator() + "PKG_ADD");
        QFile::copy(this->otherPath() + QDir::separator() + "PKG_ADD",
                    pkgFolder.path() + QDir::separator() + "PKG_ADD");

        if(QFile::exists(pkgFolder.path() + QDir::separator() + "PKG_DEL"))
            QFile::remove(pkgFolder.path() + QDir::separator() + "PKG_DEL");
        QFile::copy(this->otherPath() + QDir::separator() + "PKG_DEL",
                    pkgFolder.path() + QDir::separator() + "PKG_DEL");

        pkgFolder.mkdir("inst");
        pkgFolder.cd("inst");

        pkgFolder.mkdir("fcn");
        pkgFolder.mkdir("img");
        pkgFolder.mkdir("wnd");
        pkgFolder.mkdir("guilib");
        guiLibsFiles << "_loadIcon.m";
        for(k = 0; k < guiLibsFiles.count(); k++){
            if(QFile::exists(pkgFolder.path() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]))
                QFile::remove(pkgFolder.path() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]);
            QFile::copy(":/guilib/guilib/" + guiLibsFiles[k],
                        pkgFolder.path() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]);
            QFile(pkgFolder.path() + QDir::separator() + "guilib" + QDir::separator() + guiLibsFiles[k]).setPermissions(QFileDevice::ReadOwner | QFileDevice::WriteOwner | QFileDevice::ReadGroup);
        }
        if(pkgFolder.exists("libs"))
        {
            QDir rmLibs(pkgFolder);
            rmLibs.cd("libs");
            rmLibs.removeRecursively();
        }
        pkgFolder.mkdir("libs");
        copyFolder(this->path() + QDir::separator() + "libs", pkgFolder.path() + QDir::separator() + "libs");

        if(this->defBasePath() || this->defImgPath())
            varDef.append("[_dir, _name, _ext] = fileparts( mfilename('fullpathext') );");
        if(this->defBasePath())
            varDef.append("global _" + this->name() + "BasePath = strtrunc(_dir, length(_dir) - 4);");
        if(this->defImgPath())
            varDef.append("global _" + this->name() + "ImgPath = [ strtrunc(_dir, length(_dir) - 4) filesep() 'img'];");
        for(it = items.begin(); it != items.end(); it++)
        {
            (*it)->setVariableDefinitions(varDef);
            (*it)->exportAsScript(pkgFolder.path());
            if((*it) == this->vMainDlgName)
            {
                QFile f;
                f.setFileName(pkgFolder.path() + QDir::separator() + this->name() + ".m");
                f.open(QIODevice::WriteOnly | QIODevice::Text);
                QTextStream txt(&f);
                txt.setCodec(mainWnd::getAppConfig()->getOutputCodec().toStdString().c_str());

                txt << "## -*- texinfo -*-" << endl;
                txt << "## @deftypefn  {} {@var{ret} =} " + this->name() + " (varargin)" << endl;
                txt << "##" << endl;
                txt << "## Create and show the main dialog, return a struct as representation of dialog." << endl;
                txt << "##" << endl;
                txt << "## @end deftypefn" << endl;
                txt << "function ret = " + this->name() + "(varargin)" << endl;
                /*   Warning messages supressions   */
                if(this->disableAllWarnings)
                    txt << "  warning(\"off\");" << endl;
                for(k = 0; k < this->disabledWarningMessages.count(); k++){
                    warnMsg = this->disabledWarningMessages[k].replace("\"", "\\\"");
                    txt << "  warning(\"off\", \"" + warnMsg + "\");" << endl;
                }
                /* End warning messages supressions */
                txt << "  [dir, name, ext] = fileparts( mfilename('fullpathext') );" << endl;
                if(mainWnd::getAppConfig()->getForceOutputCodec()){
                    txt << "  _initialCodec = __mfile_encoding__;"  << endl;
                    txt << "  __mfile_encoding__(\"" + mainWnd::getAppConfig()->getOutputCodec() + "\");"  << endl;
                    txt << "  clear (\"functions\");"  << endl;
                }
                if(this->defBasePath())
                    txt << "  global _" << this->name() << "BasePath = dir;" << endl;
                if(this->defImgPath())
                    txt << "  global _" << this->name() << "ImgPath = [dir filesep() 'img'];" << endl;
                txt << "  addpath( [dir filesep() \"img\" ]);" << endl;
                txt << "  addpath( [dir filesep() \"fcn\" ]);" << endl;
                txt << "  addpath( [dir filesep() \"libs\" ]);" << endl;
                txt << "  addpath( [dir filesep() \"wnd\" ]);" << endl;
                txt << "  addpath( [dir filesep() \"guilib\" ]);" << endl;
                txt << "  " << (*it)->getName() << "(varargin{:});" << endl;
                if(mainWnd::getAppConfig()->getForceOutputCodec()){
                    txt << "  __mfile_encoding__(_initialCodec);"  << endl;
                }
                txt << "end" << endl;
                f.close();
            }
        }
        /* The project structure has made, now we must compress the folder */
        pkgFolder.cdUp();
        pkgFolder.cdUp();

        mainWnd::getPkgGenWnd()->startGen(pkgFolder.path(), this->name());
    }
}


void guiProject::save()
{
    storageManager::saveProject(this);
    mainWnd::getMainWnd()->updateRecentPrj();
    QList<projectItem *>::iterator it;
    for(it = items.begin(); it != items.end(); it++)
        (*it)->save();
}

void guiProject::disableFSWatch(void){    
    disconnect(fsw, SIGNAL(directoryChanged(QString)), this, SLOT(prjDirChanged(QString)));
    fsw->removePath(this->fcnPath());
    fsw->removePath(this->guiPath());
    fsw->removePath(this->imgPath());
    fsw->removePath(this->otherPath());

}

void guiProject::enableFSWatch(void){
    fsw->addPath(this->fcnPath());
    fsw->addPath(this->guiPath());
    fsw->addPath(this->imgPath());
    fsw->addPath(this->otherPath());
    connect(fsw, SIGNAL(directoryChanged(QString)), this, SLOT(prjDirChanged(QString)));
}


void guiProject::prjDirChanged(QString path){
    QDir changedPath;
    QString fileName;
    QStringList fileList;
    QFileInfo fileInfo;
    bool fileIsOpen;
    int i, k;
    bool newFile;
    QList<projectItem *>::iterator it;
    QList<projectItem *> toRemove;
    projectItem *itemToDel;
    if(path == this->fcnPath()){
        changedPath.setPath(path);
        fileList = changedPath.entryList(QDir::NoDotAndDotDot | QDir::Files);
        for(i = 0; i < fileList.count(); i++)
        {
            newFile = true;
            fileName = path + QDir::separator() + fileList[i];
            fileInfo.setFile(fileName);
            for(it = this->items.begin(); this->items.end() != it; it++){
                if((*it)->documentHaveName){
                    if((*it)->documentName == fileName){
                        newFile = false;
                        if((*it)->getLastModified() != fileInfo.lastModified()){
                            /* The file has change outside of editor */
                            dlgFileChanged fc(fileName, this);
                            /* What do you want to do? */
                            fc.exec();
                            /* Overwrite the file */
                            if(fc.result()== fcOverwrite){
                                (*it)->save();
                            }
                            /* Reload the file content*/
                            else {
                                /* Check if file is open in tab widget */
                                fileIsOpen = false;
                                for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                                    fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == (*it)->getName();
                                    if(fileIsOpen){
                                        this->wdgPrj->ui->docTabs->removeTab(k);
                                    }
                                }
                                (*it)->load(fileName);
                                if(fileIsOpen){
                                    int tabNro = this->wdgPrj->ui->docTabs->addTab(((childWndMFile *)(*it)), ((childWndMFile *)(*it))->getName());
                                    this->wdgPrj->ui->docTabs->setCurrentIndex(tabNro);
                                }
                            }
                        }
                    }
                }
            }
            /* if is a new file, add this to the project */
            if(newFile){
                childWndMFile  * dlg = new childWndMFile(this->wdgPrj->ui->docTabs, this);
                dlg->load(fileName);
                items.push_back(dlg);
            }
        }
        /* search for removed files */
        for(it = this->items.begin(); this->items.end() != it; it++){
            if((*it)->documentHaveName){
                if(!QFile::exists((*it)->documentName)){
                    dlgFileDeleted fd((*it)->documentName, this);
                    fd.exec();
                    if(fd.result() == fdDiscard){
                        toRemove.push_back((*it));
                    }
                    else if(fd.result() == fdSave){
                        (*it)->save();
                    }
                }
            }
        }
        while(toRemove.count() > 0){
            itemToDel = toRemove.takeFirst();
            fileIsOpen = false;
            for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == itemToDel->getName();
                if(fileIsOpen){
                    this->wdgPrj->ui->docTabs->removeTab(k);
                }
            }
            this->items.removeAll(itemToDel);
            delete itemToDel;
        }
    }
    else if(path == this->guiPath()){
        changedPath.setPath(path);
        fileList = changedPath.entryList(QDir::NoDotAndDotDot | QDir::Files);
        for(i = 0; i < fileList.count(); i++)
        {
            newFile = true;
            fileName = path + QDir::separator() + fileList[i];
            fileInfo.setFile(fileName);
            for(it = this->items.begin(); this->items.end() != it; it++){
                if((*it)->documentHaveName){
                    if((*it)->documentName == fileName){
                        newFile = false;
                        if((*it)->getLastModified() != fileInfo.lastModified()){
                            /* The file has change outside of editor */
                            dlgFileChanged fc(fileName, this);
                            /* What do you want to do? */
                            fc.exec();
                            /* Overwrite the file */
                            if(fc.result()== fcOverwrite){
                                (*it)->save();
                            }
                            /* Reload the file content*/
                            else {
                                /* Check if file is open in tab widget */
                                fileIsOpen = false;
                                for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                                    fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == (*it)->getName();
                                    if(fileIsOpen){
                                        this->wdgPrj->ui->docTabs->removeTab(k);
                                    }
                                }                                
                                (*it)->load(fileName);
                                if(fileIsOpen){
                                    int tabNro = this->wdgPrj->ui->docTabs->addTab(((childWndDlg *)(*it)), ((childWndDlg *)(*it))->getName());
                                    this->wdgPrj->ui->docTabs->setCurrentIndex(tabNro);
                                }
                            }
                        }
                    }
                }
            }
            /* if is a new file, add this to the project */
            if(newFile){
                childWndDlg  * dlg = new childWndDlg(this->wdgPrj->ui->docTabs,
                                                     mainWnd::getPropPan(),
                                                     this);
                dlg->load(fileName);
                items.push_back(dlg);
            }
        }
        /* search for removed files */
        for(it = this->items.begin(); this->items.end() != it; it++){
            if((*it)->documentHaveName){
                if(!QFile::exists((*it)->documentName)){
                    dlgFileDeleted fd((*it)->documentName, this);
                    fd.exec();
                    if(fd.result() == fdDiscard){
                        toRemove.push_back((*it));
                    }
                    else if(fd.result() == fdSave){
                        (*it)->save();
                    }
                }
            }
        }
        while(toRemove.count() > 0){
            itemToDel = toRemove.takeFirst();
            fileIsOpen = false;
            for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == itemToDel->getName();
                if(fileIsOpen){
                    this->wdgPrj->ui->docTabs->removeTab(k);
                }
            }
            this->items.removeAll(itemToDel);
            delete itemToDel;
        }
    }
    else if (path == this->imgPath()){
        changedPath.setPath(path);
        fileList = changedPath.entryList(QDir::NoDotAndDotDot | QDir::Files);
        for(i = 0; i < fileList.count(); i++)
        {
            newFile = true;
            fileName = path + QDir::separator() + fileList[i];
            fileInfo.setFile(fileName);
            for(it = this->items.begin(); this->items.end() != it; it++){
                if((*it)->documentHaveName){
                    if((*it)->documentName == fileName){
                        newFile = false;
                        if((*it)->getLastModified() != fileInfo.lastModified()){
                            /* The file has change outside of editor */
                            dlgFileChanged fc(fileName, this);
                            /* What do you want to do? */
                            fc.exec();
                            /* Overwrite the file */
                            if(fc.result()== fcOverwrite){
                                (*it)->save();
                            }
                            /* Reload the file content*/
                            else {
                                /* Check if file is open in tab widget */
                                fileIsOpen = false;
                                for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                                    fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == (*it)->getName();
                                    if(fileIsOpen){
                                        this->wdgPrj->ui->docTabs->removeTab(k);
                                    }
                                }
                                (*it)->load(fileName);
                                if(fileIsOpen){
                                    int tabNro = this->wdgPrj->ui->docTabs->addTab(((childWndImg *)(*it)), ((childWndImg *)(*it))->getName());
                                    this->wdgPrj->ui->docTabs->setCurrentIndex(tabNro);
                                }
                            }
                        }
                    }
                }
            }
            /* if is a new file, add this to the project */
            if(newFile){
                childWndImg  * dlg = new childWndImg(this->wdgPrj->ui->docTabs, this, fileName);
                dlg->load(fileName);
                items.push_back(dlg);
            }
        }
        /* search for removed files */
        for(it = this->items.begin(); this->items.end() != it; it++){
            if((*it)->documentHaveName){
                if(!QFile::exists((*it)->documentName)){
                    dlgFileDeleted fd((*it)->documentName, this);
                    fd.exec();
                    if(fd.result() == fdDiscard){
                        toRemove.push_back((*it));
                    }
                    else if(fd.result() == fdSave){
                        (*it)->save();
                    }
                }
            }
        }
        while(toRemove.count() > 0){
            itemToDel = toRemove.takeFirst();
            fileIsOpen = false;
            for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == itemToDel->getName();
                if(fileIsOpen){
                    this->wdgPrj->ui->docTabs->removeTab(k);
                }
            }
            this->items.removeAll(itemToDel);
            delete itemToDel;
        }
    }
    else if(path == this->otherPath()){
        changedPath.setPath(path);
        fileList = changedPath.entryList(QDir::NoDotAndDotDot | QDir::Files);
        for(i = 0; i < fileList.count(); i++)
        {
            newFile = true;
            fileName = path + QDir::separator() + fileList[i];
            fileInfo.setFile(fileName);
            for(it = this->items.begin(); this->items.end() != it; it++){
                if((*it)->documentHaveName){
                    if((*it)->documentName == fileName){
                        newFile = false;
                        if((*it)->getLastModified() != fileInfo.lastModified()){
                            /* The file has change outside of editor */
                            dlgFileChanged fc(fileName, this);
                            /* What do you want to do? */
                            fc.exec();
                            /* Overwrite the file */
                            if(fc.result()== fcOverwrite){
                                (*it)->save();
                            }
                            /* Reload the file content*/
                            else {
                                /* Check if file is open in tab widget */
                                fileIsOpen = false;
                                for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                                    fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == (*it)->getName();
                                    if(fileIsOpen){
                                        this->wdgPrj->ui->docTabs->removeTab(k);
                                    }
                                }
                                (*it)->load(fileName);
                                if(fileIsOpen){
                                    int tabNro = this->wdgPrj->ui->docTabs->addTab(((childWndTextFile *)(*it)), ((childWndTextFile *)(*it))->getName());
                                    this->wdgPrj->ui->docTabs->setCurrentIndex(tabNro);
                                }
                            }
                        }
                    }
                }
            }
            /* if is a new file, add this to the project */
            if(newFile){
                childWndTextFile  * dlg = new childWndTextFile(this->wdgPrj->ui->docTabs, this);
                dlg->load(fileName);
                items.push_back(dlg);
            }
        }
        /* search for removed files */
        for(it = this->items.begin(); this->items.end() != it; it++){
            if((*it)->documentHaveName){
                if(!QFile::exists((*it)->documentName)){
                    dlgFileDeleted fd((*it)->documentName, this);
                    fd.exec();
                    if(fd.result() == fdDiscard){
                        toRemove.push_back((*it));
                    }
                    else if(fd.result() == fdSave){
                        (*it)->save();
                    }
                }
            }
        }
        while(toRemove.count() > 0){
            itemToDel = toRemove.takeFirst();
            fileIsOpen = false;
            for(k = 0; (k < this->wdgPrj->ui->docTabs->count()) && (!fileIsOpen); k++){
                fileIsOpen = this->wdgPrj->ui->docTabs->tabText(k) == itemToDel->getName();
                if(fileIsOpen){
                    this->wdgPrj->ui->docTabs->removeTab(k);
                }
            }
            this->items.removeAll(itemToDel);
            delete itemToDel;
        }
    }
    this->wdgPrj->updateTreeNames();
}
