#include "setupwizard.h"
#include <QVBoxLayout>
#include <QFileDialog>
#include <QFontDialog>
#include <QHBoxLayout>
#include <QMessageBox>
#include <QDebug>
#include <QString>
#include <QFont>

firstPage::firstPage(QWidget *parent, configSettings *appCfg __attribute__((unused))):QWizardPage(parent){
    setTitle(tr("guiEditor configuration"));

    label = new QLabel(tr("This wizard setup basic options to work with guiEditor. "
                          "Through the following options you will be able to select "
                          "the language of the graphical interface as well as "
                          "different options related to the tools required for "
                          "the configuration of the packages."));
    label->setWordWrap(true);
    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(label);
    setLayout(layout);
}

void firstPage::initializePage(void){
    wizard()->setButtonText(QWizard::CancelButton, tr("Cancelar"));
    wizard()->setButtonText(QWizard::NextButton, tr("Siguiente"));
    wizard()->setButtonText(QWizard::BackButton, tr("Anterior"));
    wizard()->setButtonText(QWizard::FinishButton, tr("Finalizar"));
}

secondPage::secondPage(QWidget *parent, configSettings *appCfg):QWizardPage(parent){
    setTitle(tr("Interface language"));

    cmbLanguage = new QComboBox();
    cmbLanguage->addItem("Spanish");
    cmbLanguage->addItem("English");
    cmbLanguage->setCurrentText(appCfg->getLanguage());

    registerField("language", cmbLanguage, "currentText");

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(cmbLanguage);
    setLayout(layout);
}

thirdPage::thirdPage(QWidget *parent, configSettings *appCfg):QWizardPage(parent){
    QByteArray snapPath = qgetenv("SNAP");
    setTitle(tr("GNU Octave path"));
    topMsg = new QLabel();
    octPath = new QLabel();
    qtPlugPath = new QLabel();

    changeOctPath = new QPushButton();
    changePlugPath = new QPushButton();
    clearPlugPath = new QPushButton();
    if(snapPath.length() > 0){
        changeOctPath->setEnabled(false);
        changePlugPath->setEnabled(false);
        clearPlugPath->setEnabled(false);
    }
    topMsg->setText(tr("In order to run the project from the GUI editor,"
                       "you need to specify the path where the GNU Octave "
                       "executable file is located."));
    topMsg->setWordWrap(true);
    midMsg = new QLabel();
    midMsg->setText(tr("In some cases, depending on the version and/or "
                       "configuration used, it is necessary to indicate "
                       "the path where the Qt plugins included in the GNU"
                       "Octave distribution are stored. If this is the "
                       "case, you can indicate from this option the path "
                       "to use. If you are not sure about this option, you "
                       "may not need to enter a value. \n Actual value of QT_PLUGIN_PATH:"));
    midMsg->setWordWrap(true);

    octPath->setText(appCfg->getOctavePath());
    octPath->setToolTip(octPath->text());

    qtPlugPath->setText(appCfg->getQt5PlugginPath());
    qtPlugPath->setToolTip(qtPlugPath->text());

    changeOctPath->setText("...");
    changePlugPath->setText("...");
    clearPlugPath->setText("X");
    clearPlugPath->setToolTip(tr("Clear the QT_PLUGIN_PATH value"));
    changeOctPath->setMaximumWidth(32);
    changePlugPath->setMaximumWidth(32);
    clearPlugPath->setMaximumWidth(32);

    connect(changeOctPath, SIGNAL(clicked(bool)), this, SLOT(changeOctPath_click(bool)));
    connect(changePlugPath, SIGNAL(clicked(bool)), this, SLOT(changePlugPath_click(bool)));
    connect(clearPlugPath, SIGNAL(clicked(bool)), this, SLOT(clearPlugPath_click(bool)));

    QHBoxLayout *hl = new QHBoxLayout;
    hl->addWidget(new QLabel(tr("GNU Octave path:")));
    hl->addWidget(octPath);
    hl->addItem(new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hl->addWidget(changeOctPath);
    QVBoxLayout * vl = new QVBoxLayout();
    vl->addWidget(topMsg);
    vl->addLayout(hl);
    vl->addWidget(midMsg);
    hl = new QHBoxLayout;
    hl->addWidget(new QLabel(tr("Qt plugin path:")));
    hl->addWidget(qtPlugPath);
    hl->addItem(new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
    hl->addWidget(changePlugPath);
    hl->addWidget(clearPlugPath);
    vl->addLayout(hl);
    setLayout(vl);
    registerField("octavePath", octPath, "text");
    registerField("pluginPath", qtPlugPath, "text");
}

void thirdPage::changeOctPath_click(bool checked __attribute__((unused))){
   QString path;
   path = field("octavePath").toString();
   QString newPath = QFileDialog::getOpenFileName(this, tr("Octave file path"), path, tr("Octave binary(octave*);;All files (*.*);;"));
   if(!newPath.isNull()){
       octPath->setText(newPath);
       octPath->setToolTip(newPath);
   }
}

void thirdPage::changePlugPath_click(bool checked  __attribute__((unused))){
    QString path;
    path = field("pluginPath").toString();
    QString newPath = QFileDialog::getExistingDirectory(this, tr("Qt Plugin path"), path);
    if(!newPath.isNull()){
        qtPlugPath->setText(newPath);
        qtPlugPath->setToolTip(newPath);
    }
}

void thirdPage::clearPlugPath_click(bool checked  __attribute__((unused))){
    qtPlugPath->setText(tr("[Not used]"));
    qtPlugPath->setToolTip(tr("[Not used]"));
}

fourthPage::fourthPage(QWidget *parent, configSettings *appCfg):QWizardPage(parent){
  setTitle(tr("Applications required to generate packages"));
  QByteArray snapPath = qgetenv("SNAP");
  topMsg = new QLabel();
  topMsg->setText(tr("In order to generate packages, it is necessary to have two "
                  "applications installed on the system and that can be executed "
                  "from the command line. \n"
                  "These are gzip and tar, both of which are available in GNU Linux "
                  "distributions and there are different versions for Microsoft Windows.\n"
                  "In this window you should indicate the path where each of them "
                  "is located."));
  topMsg->setWordWrap(true);
  gzipPath = new QLabel();
  midMsg = new QLabel();
  tarPath = new QLabel();
  changeGzipPath = new QPushButton();
  changeTarPath = new QPushButton();

  gzipPath->setText(appCfg->getGzipPath());
  gzipPath->setToolTip(gzipPath->text());

  tarPath->setText(appCfg->getTarPath());
  tarPath->setToolTip(tarPath->text());

  changeTarPath->setText("...");
  changeTarPath->setMaximumWidth(32);
  changeGzipPath->setText("...");
  changeGzipPath->setMaximumWidth(32);

  if(snapPath.length()>0){
      changeTarPath->setEnabled(false);
      changeGzipPath->setEnabled(false);
  }
  connect(changeGzipPath, SIGNAL(clicked(bool)), this, SLOT(changeGzipPath_click(bool)));
  connect(changeTarPath, SIGNAL(clicked(bool)), this, SLOT(changeTarPath_click(bool)));

  QVBoxLayout * vl = new QVBoxLayout();
  vl->addWidget(topMsg);
  QHBoxLayout * hl = new QHBoxLayout();
  hl->addWidget(new QLabel(tr("GZip path:")));
  hl->addWidget(gzipPath);
  hl->addItem(new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
  hl->addWidget(changeGzipPath);
  vl->addLayout(hl);

  hl = new QHBoxLayout();
  hl->addWidget(new QLabel(tr("Tar path:")));
  hl->addWidget(tarPath);
  hl->addItem(new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum));
  hl->addWidget(changeTarPath);

  vl->addLayout(hl);
  setLayout(vl);
  registerField("gzipPath", gzipPath, "text");
  registerField("tarPath", tarPath, "text");
}

void fourthPage::changeGzipPath_click(bool checked __attribute__((unused))){
    QString path;
    path = field("gzipPath").toString();
    QString newPath = QFileDialog::getOpenFileName(this, tr("gzip file path"), path, tr("gzip binary(gzip*);;All files (*.*);;"));
    if(!newPath.isNull()){
        gzipPath->setText(newPath);
        gzipPath->setToolTip(newPath);
    }
}

void fourthPage::changeTarPath_click(bool checked __attribute__((unused))){
    QString path;
    path = field("tarPath").toString();
    QString newPath = QFileDialog::getOpenFileName(this, tr("tar file path"), path, tr("tar binary(tar*);;All files (*.*);;"));
    if(!newPath.isNull()){
        tarPath->setText(newPath);
        tarPath->setToolTip(newPath);
    }
}

fifthPage::fifthPage(QWidget *parent, configSettings *appCfg):QWizardPage(parent){
    setTitle(tr("Font types and sizes"));
    topMsg = new QLabel();
    guiFName = new QLabel();
    guiFSize = new QLabel();
    midMsg = new QLabel();
    srcFName = new QLabel();
    srcFSize = new QLabel();

    topMsg->setText(tr("You can select the font that will be used"
                    " as default in the controls of your applications. "
                    "It is recommended that it be a standard font, so"
                    " that it is available on most systems."));
    topMsg->setWordWrap(true);

    midMsg->setText(tr("Similarly, you can indicate your preference "
                    "regarding the font used in source code editing "
                    "spaces. This option will not impact the applications "
                    "you develop, only the way you view the source "
                    "code you write."));
    midMsg->setWordWrap(true);

    guiFName->setText(appCfg->getCtrlFont().family());
    guiFSize->setText(QString::number(appCfg->getCtrlFont().pointSize()));

    srcFName->setText(appCfg->getEditorFont().family());
    srcFSize->setText(QString::number(appCfg->getEditorFont().pointSize()));

    changeGuiFont = new QPushButton();
    changeSrcFont = new QPushButton();

    connect(changeGuiFont, SIGNAL(clicked(bool)), this, SLOT(changeGuiFont_click(bool)));
    connect(changeSrcFont, SIGNAL(clicked(bool)), this, SLOT(changeSrcFont_click(bool)));

    changeGuiFont->setText("...");
    changeGuiFont->setMaximumWidth(32);
    changeGuiFont->setToolTip(tr("Change font used in gui controls"));
    changeSrcFont->setText("...");
    changeSrcFont->setMaximumWidth(32);
    changeSrcFont->setToolTip(tr("Change font used in source code editor window"));

    QVBoxLayout * vl = new QVBoxLayout();
    vl->addWidget(topMsg);
    QHBoxLayout * hl = new QHBoxLayout();
    hl->addWidget(guiFName);
    hl->addWidget(new QLabel(", "));
    hl->setSizeConstraint(QLayout::SetMinimumSize);
    hl->addWidget(guiFSize);
    QSpacerItem * hs1 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    hl->addItem(hs1);
    hl->addWidget(changeGuiFont);
    vl->addLayout(hl);
    vl->addWidget(midMsg);
    hl = new QHBoxLayout();
    hl->addWidget(srcFName);
    hl->addWidget(new QLabel(", "));
    hl->addWidget(srcFSize);
    hs1 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    hl->addItem(hs1);
    hl->addWidget(changeSrcFont);
    vl->addLayout(hl);
    registerField("guiFName", guiFName, "text");
    registerField("guiFSize", guiFSize, "text");
    registerField("srcFName", srcFName, "text");
    registerField("srcFSize", srcFSize, "text");
    setLayout(vl);
}

void fifthPage::changeGuiFont_click(bool checked __attribute__((unused))){
    bool ok;
    QString fontName = field("guiFName").toString();
    int fontSize = field("guiFSize").toString().toInt();
    QFont selFont = QFontDialog::getFont(&ok, QFont(fontName, fontSize), this, tr("Font name and size for gui controls"));
    if(ok){
        guiFName->setText(selFont.family());
        guiFSize->setText(QString::number(selFont.pointSize()));
    }
}

void fifthPage::changeSrcFont_click(bool checked __attribute__((unused))){
    bool ok;
    QString fontName = field("srcFName").toString();
    int fontSize = field("srcFSize").toString().toInt();
    QFont selFont = QFontDialog::getFont(&ok, QFont(fontName, fontSize), this, tr("Font name and size for source code edit window"));
    if(ok){
        srcFName->setText(selFont.family());
        srcFSize->setText(QString::number(selFont.pointSize()));
    }
}

sixthPage::sixthPage(QWidget *parent, configSettings *appCfg):QWizardPage(parent){
    setTitle(tr("Script generation options"));
    topMsg = new QLabel();
    outEncoding = new QComboBox();
    midMsg = new QLabel();
    forceCodec = new QCheckBox();
    cmbTitle = new QLabel();

    topMsg->setText(tr("The generated script files are text files. "
                       "These can include special characters which, "
                       "depending on the output codec used, can "
                       "achieve different representations. With the "
                       "next option you can choose which codec will be"
                       "used for the generation of the script files. If "
                       "you don't have a preference, perhaps UTF-8 is a"
                       " suitable option."));
    topMsg->setWordWrap(true);
    cmbTitle->setText(tr("Output codec:"));
    outEncoding->addItem("Big5");
    outEncoding->addItem("Big5-HKSCS");
    outEncoding->addItem("CP949");
    outEncoding->addItem("EUC-JP");
    outEncoding->addItem("EUC-KR");
    outEncoding->addItem("GB18030");
    outEncoding->addItem("HP-ROMAN8");
    outEncoding->addItem("IBM 850");
    outEncoding->addItem("IBM 866");
    outEncoding->addItem("IBM 874");
    outEncoding->addItem("ISO 2022-JP");
    outEncoding->addItem("ISO 8859-1");
    outEncoding->addItem("ISO 8859-2");
    outEncoding->addItem("ISO 8859-3");
    outEncoding->addItem("ISO 8859-4");
    outEncoding->addItem("ISO 8859-5");
    outEncoding->addItem("ISO 8859-6");
    outEncoding->addItem("ISO 8859-7");
    outEncoding->addItem("ISO 8859-8");
    outEncoding->addItem("ISO 8859-9");
    outEncoding->addItem("ISO 8859-10");
    outEncoding->addItem("ISO 8859-13");
    outEncoding->addItem("ISO 8859-14");
    outEncoding->addItem("ISO 8859-15");
    outEncoding->addItem("ISO 8859-16");
    outEncoding->addItem("Iscii-Bng");
    outEncoding->addItem("Iscii-Dev");
    outEncoding->addItem("Iscii-Gjr");
    outEncoding->addItem("Iscii-Knd");
    outEncoding->addItem("Iscii-Mlm");
    outEncoding->addItem("Iscii-Ori");
    outEncoding->addItem("Iscii-Pnj");
    outEncoding->addItem("Iscii-Tlg");
    outEncoding->addItem("Iscii-Tml");
    outEncoding->addItem("KOI8-R");
    outEncoding->addItem("KOI8-U");
    outEncoding->addItem("Macintosh");
    outEncoding->addItem("Shift-JIS");
    outEncoding->addItem("TIS-620");
    outEncoding->addItem("TSCII");
    outEncoding->addItem("UTF-8");
    outEncoding->addItem("UTF-16");
    outEncoding->addItem("UTF-16BE");
    outEncoding->addItem("UTF-16LE");
    outEncoding->addItem("UTF-32");
    outEncoding->addItem("UTF-32BE");
    outEncoding->addItem("UTF-32LE");
    outEncoding->addItem("Windows-1250");
    outEncoding->addItem("Windows-1251");
    outEncoding->addItem("Windows-1252");
    outEncoding->addItem("Windows-1253");
    outEncoding->addItem("Windows-1254");
    outEncoding->addItem("Windows-1255");
    outEncoding->addItem("Windows-1256");
    outEncoding->addItem("Windows-1257");
    outEncoding->addItem("Windows-1258");
    outEncoding->setCurrentText(appCfg->getOutputCodec());

    midMsg->setText(tr("Just as a codec is used to generate the script "
                       "files, it may be necessary to configure the "
                       "same codec to be used when executing the scripts. "
                       "The following option, if checked, adds code to the "
                       "beginning of the application so that scripts are "
                       "forced to render with the codec used for generation. "
                       "At the end of the application, an attempt will be "
                       "made to restore the originally configured code."));
    midMsg->setWordWrap(true);
    forceCodec->setText(tr("Force codec settings"));
    forceCodec->setChecked(appCfg->getForceOutputCodec());

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(topMsg);
    QHBoxLayout *hl = new QHBoxLayout;
    QSpacerItem * hs1 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    hl->addItem(hs1);
    hl->addWidget(cmbTitle);
    hl->addWidget(outEncoding);
    QSpacerItem * hs2 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    hl->addItem(hs2);
    layout->addLayout(hl);


    layout->addWidget(midMsg);
    hl = new QHBoxLayout;
    QSpacerItem * hs3 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    hl->addItem(hs3);
    hl->addWidget(forceCodec);
    QSpacerItem * hs4 = new QSpacerItem(1, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    hl->addItem(hs4);
    layout->addLayout(hl);

    setLayout(layout);
    registerField("outEncoding", outEncoding, "currentText");
    registerField("forceCodec", forceCodec);
}

lastPage::lastPage(QWidget *parent, configSettings *appCfg __attribute__((unused))):QWizardPage(parent){
    setTitle(tr("Configuration finished!"));
    topMsg = new QLabel(tr("If in the future you want to modify "
                           "any of the established options, you "
                           "can do it from the File->Config Wizard "
                           "option of the main menu."
                           "If you made changes from the defaults, "
                           " please restart the application after "
                           "clicking the \"Finish\" button."));
    topMsg->setWordWrap(true);

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(topMsg);
    setLayout(layout);
}


setupWizard::setupWizard(configSettings *appCfg, QWidget *parent):QWizard(parent)
{
  addPage(new firstPage(this, appCfg));
  addPage(new secondPage(this, appCfg));
  addPage(new thirdPage(this, appCfg));
  addPage(new fourthPage(this, appCfg));
  addPage(new fifthPage(this, appCfg));
  addPage(new sixthPage(this, appCfg));
  addPage(new lastPage(this, appCfg));
  setWindowTitle(tr("Config wizard"));
  cfg = appCfg;
}

void setupWizard::accept(void)
{
    cfg->setLanguage(field("language").toString());
    cfg->setOctavePath(field("octavePath").toString());
    cfg->setQt5PlugginPath(field("pluginPath").toString());
    cfg->setGzipPath(field("gzipPath").toString());
    cfg->setTarPath(field("tarPath").toString());
    cfg->setCtrlFont(QFont(field("guiFName").toString(), field("guiFSize").toString().toInt()));
    cfg->setEditorFont(QFont(field("srcFName").toString(), field("srcFSize").toString().toInt()));
    cfg->setOutputCodec(field("outEncoding").toString());
    cfg->setForceOutputCodec(field("forceCodec").toBool());
    cfg->saveCfg();
    QDialog::accept();
}
